#!/usr/bin/python2
#-*-coding:UTF-8-*-
##
# Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
# 
# This software is published by the license of CPU-OS License,you can use and 
# distribute this software under this License. See CPU-OS License for more detail.
#
# You should have receive a copy of CPU-OS License. If not, please contact us 
# by email <support_os@cpu-os.ac.cn>  
# 
##

from __future__ import print_function
import sys
import os
import shutil
import pdb

LSB_RELEASE_FILE='/etc/lsb-release'
OS_RELEASE_FILE='/etc/os-release'
ISSUE_FILE='/etc/issue'
ISSUENET_FILE='/etc/issue.net'
MOTD_FILE='/etc/motd'
SYSTEM_RELEASE_FILE='/etc/system-release'

# for test
#LSB_RELEASE_FILE='/tmp/lsb-release'
#OS_RELEASE_FILE='/tmp/os-release'
#ISSUE_FILE='/tmp/issue'
#ISSUENET_FILE='/tmp/issue.net'


distinfo = {}

def get_os_release():
    cdos_info='/etc/nfs/info'
    if os.path.exists(cdos_info):
        try:
            with open(cdos_info) as info_file:
                for line in info_file:
                    line = line.strip()
                    if not line:
                        continue
                    if line.startswith('#'):
                        continue
                    # Skip invalid lines
                    if not '=' in line:
                        continue
                    var, arg = line.split('=', 1)
                    if var.startswith('NAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('NAME_EN'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('VERSION'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('VERSION_ID'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('MINOR_VERSION'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('ARCH'):
                        distinfo[var] = arg.strip()
                    if var.startswith('FEATURE_NAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('ID'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('ID_LIKE'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('CODENAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('VERSION_CODENAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('DESKTOP'):
                        distinfo[var] = arg.strip()
                    if var.startswith('DESCRIPTION'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('KERNEL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                        else:
                            f = open('/proc/sys/kernel/osrelease')
                            version = f.readline().split('\n')[0]
                            distinfo[var] = version.strip()
                    if var.startswith('HOME_URL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('SUPPORT_URL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('BUG_REPORT_URL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_ID'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_RELEASE'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_CODENAME'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_DESCRIPTION'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('UPSTREAM_KERNEL'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith('GRUB_TITLE'):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
                    if var.startswith("RELEASE_TYPE"):
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg: # Ignore empty arguments
                            distinfo[var] = arg.strip()
        except IOError as msg:
            print('Unable to open ' + etc_cdos_release + ':', str(msg), file=sys.stderr)
    return distinfo

def update_lsb_release_file():
    """ Update /etc/lsb_release file for This OS"""
    distrib_id = distinfo.get('ID', 'nfsdesktop')
    distrib_release = distinfo.get('VERSION_ID', 'n/a')
    distrib_codename = distinfo.get('VERSION_CODENAME', 'n/a')
    
    name_en = distinfo.get('NAME_EN', 'NFSDesktop')
    version = distinfo.get('VERSION', 'n/a')
    minor_version = distinfo.get('MINOR_VERSION', 'G210')
    distrib_description = "%s %s-%s" % (name_en, version, minor_version)

    relinfo = """DISTRIB_ID=%s
DISTRIB_RELEASE=%s
DISTRIB_CODENAME=%s
DISTRIB_DESCRIPTION=\"%s\"
""" % (distrib_id, distrib_release, distrib_codename, distrib_description)
    

    #if os.path.exists('/etc/lsb-release'):

    try:
        lsb_release_file = open(LSB_RELEASE_FILE, 'wb')
        try:
            lsb_release_file.write(relinfo)
        finally:
            lsb_release_file.close()
    except IOError as msg:
        print("Failed to open file %s :" % lsb_release_file, str(msg), file=sys.stderr) 

def update_os_release_file():
    """ Update /etc/os-release file for CDOS"""
    name = distinfo.get('NAME', '方德桌面操作系统')
    name_en = distinfo.get('NAME_EN', 'NFSDesktop')
    version = distinfo.get('VERSION', 'n/a')
    version_id = distinfo.get('VERSION_ID', 'n/a')
    minor_version = distinfo.get('MINOR_VERSION', 'G210')
    name_id = distinfo.get('ID', 'nfsdesktop')
    id_like = distinfo.get('ID_LIKE', 'n/a')

    codename = distinfo.get('CODENAME', 'cdos')
    pretty_name = "%s %s-%s" % (name_en, version, minor_version)

    version_codename = distinfo.get('VERSION_CODENAME', 'n/a')
    home_url = distinfo.get('HOME_URL', 'n/a')
    release_type = distinfo.get('RELEASE_TYPE','RETAIL')
    #version = "%s-%s-%s" % (codename, arch, fn)    
    relinfo = """NAME=\"%s\"
NAME_EN=\"%s\"
VERSION=\"%s\"
VERSION_ID=\"%s\"
MINOR_VERSION=\"%s\"
ID=%s
ID_LIKE=%s
PRETTY_NAME=\"%s\"
VERSION_CODENAME=%s
HOME_URL=\"%s\"
RELEASE_TYPE=\"%s\"
""" % (name, name_en, version, version_id, minor_version, name_id, id_like, pretty_name, version_codename, home_url, release_type)


    try:
        os_release_file = open(OS_RELEASE_FILE, 'wb')
        try:
            os_release_file.write(relinfo)
        finally:
            os_release_file.close()
    except IOError as msg:
        print("Failed to open file %s :" % os_release_file, str(msg), file=sys.stderr) 

def update_issue_files():
    """ Update /etc/issue and /etc/issue.net files"""
    desc = distinfo.get('DESCRIPTION', 'n/a')

    try:
        issue_file = open(ISSUE_FILE, 'wb')
        try:
            issue_file.write("%s \\n \l" % desc)
        finally:
            issue_file.close()
    except IOError as msg:
        print("Failed to open file %s:" % issue_file, str(msg), file=sys.stderr)


    try:
        issuenet_file = open(ISSUENET_FILE, 'wb')
        try:
            issuenet_file.write("%s" % desc)
        finally:
            issuenet_file.close()
    except IOError as msg:
        print("Failed to open file %s: " % issuenet_file, str(msg), file=sys.stderr)

def update_motd_file():
    """ Update /etc/motd file"""
    name = distinfo.get('NAME', 'n/a')

    try:
        motd_file = open(MOTD_FILE, 'wb')
        try:
            motd_file.write("\nWelcome to %s system!\n\n" % name)
        finally:
            motd_file.close()
    except IOError as msg:
        print("Failed to open file " % motd_file, str(msg), file=sys.stderr)

def update_system_release_file():
    """Update /etc/system-release"""
    codename_first, codename_last = distinfo.get('CODENAME', ' ').split('-', 1)
    relinfo = """NFSDesktop release %s (%s)""" % (codename_first, codename_last)

    try:
        system_release_file = open(SYSTEM_RELEASE_FILE, 'wb')
        try:
            system_release_file.write(relinfo)
        finally:
            system_release_file.close()
    except IOError as msg:
         print("Failed to open file " % SYSTEM_RELEASE_FILE, str(msg), file=sys.stderr)


def main():
    get_os_release()
    #pdb.set_trace()
    update_lsb_release_file()
    update_os_release_file()
    update_issue_files()
    update_motd_file()
    update_system_release_file()

    print('update base-files end.')

if __name__ == '__main__':
    main()

